/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2012 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
import com.tplan.robot.ApplicationSupport;
import com.tplan.robot.AutomatedRunnable;
import com.tplan.robot.util.Utils;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Reads the input text file and starts the processes.
 */

class ScriptExecutor {
    
    /**
     * Read the argument text file and start up a test script process for each
     * line containing a valid set of T-Plan Robot Enterprise CLI parameters.
     * @param textFile the argument text file.
     */
    public static void execute(String textFile) {
        File f = new File(textFile);

        // Verify that we can read the input text file
        if (!f.exists()) {
            System.out.println("The argument " + f.getAbsolutePath() + " file does not exist!");
            System.exit(2);
        } else if (!f.isFile()) {
            System.out.println("The argument " + f.getAbsolutePath() + " file is not a file!");
            System.exit(2);
        } else if (!f.canRead()) {
            System.out.println("This application doesn't have sufficient permissions to read the argument file of  " + f.getAbsolutePath() + "!");
            System.exit(2);
        }

        int exitCode = 0;

        try {
            // Create the Robot's core class
            ApplicationSupport support = new ApplicationSupport();

            // Process the input text file
            BufferedReader reader = new BufferedReader(new FileReader(f));
            String line;
            int number = 1;
            AutomatedRunnable runnable;

            // Create the list of the process entries. This is needed
            // for us to check the status and return the overall results.
            List<ProcessEntry> processes = new ArrayList<ProcessEntry>();
            while ((line = reader.readLine()) != null) {
                line = line.trim();
                if (!line.isEmpty() && !line.startsWith("#")) {
                    runnable = createScriptRunnable(support, line, "scriptstarter process #" + number);
                    processes.add(new ProcessEntry(runnable, line, number));
                    number++;
                }
            }

            // Start the processes
            if (!processes.isEmpty()) {
                List<ProcessEntry> copy = new ArrayList<ProcessEntry>();
                long startTime = System.currentTimeMillis();
                for (ProcessEntry process : processes) {
                    process.thread.start();
                }

                // Keep checking if the threads are still running
                while (!processes.isEmpty()) {
                    try {
                        Thread.sleep(50);
                    } catch (InterruptedException ex) {
                        break;
                    }
                    copy.addAll(processes);
                    for (ProcessEntry process : copy) {
                        if (!process.thread.isAlive()) {  // Is the thread still running?
                            int code = process.runnable.getExitCode();
                            if (code != 0) {
                                exitCode = code;
                            }
                            processes.remove(process);
                        }
                    }
                }

                System.out.println("\nTotal time: "+
                        Utils.getTimePeriodForDisplay(System.currentTimeMillis() - startTime, true, true, true));
            } else {
                System.out.println("\nThe argument text file contains no valid start commands!");
                exitCode = 3;
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
        System.exit(exitCode);
    }

    /**
     * Create a process runnable from the given text line.
     * @param support the Robot's core class.
     * @param line the line containing the process CLI parameters.
     * @param threadID an ID for the thread.
     * @return the runnable.
     */
    private static AutomatedRunnable createScriptRunnable(ApplicationSupport support, String line, String threadID) {
        // First split the line to tokens
        String cliParams[] = Utils.splitCli(line);

        // Ensure that the -n/--nodisplay option is specified
        List<String> l = new ArrayList<String>(Arrays.asList(cliParams));
        if (!l.contains("-n") && !l.contains("--nodisplay")) {
            l.add("-n");
            cliParams = l.toArray(new String[l.size()]);
        }

        // Start the specified script in a standalone thread
        AutomatedRunnable runnable = support.createAutomatedRunnable(threadID, cliParams, System.out, false);
        return runnable;
    }

    /**
     * Entry of a single test process.
     */
    private static class ProcessEntry {

        final Thread thread;
        final AutomatedRunnable runnable;
        final String line;
        final int number;

        ProcessEntry(AutomatedRunnable r, String l, int n) {
            this.thread = new Thread(r);
            this.runnable = r;
            this.line = l;
            this.number = n;
        }
    }
}
